/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner;

import de.frankmuenster.jameica.qrscanner.cliurlhandler.CliUrlHandler;
import de.frankmuenster.jameica.qrscanner.gui.navigation.QrCodeScannerExtension;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.StartupParams;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Haupt-Klasse des jameica.qrscanner Plug-ins.
 * 
 * @author munstfr
 * @version 1.2
 * 
 */
public class QrCodeScanner extends AbstractPlugin {

	/**
	 * Standardkonsturktor
	 */
	public QrCodeScanner() {
		super();
	}

	/**
	 * Methode die beim Starten des Plug-Ins geladen wird
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
	 */
	public void init() throws ApplicationException {
		super.init();

		ladeHandler();
		new WebcamWatchdog();

		// Menu des Jameica-'Plugins' erweitern
		ExtensionRegistry.register(new QrCodeScannerExtension(), "jameica.start");

		// Check ob mittels bank:// URL gestartet
		final StartupParams startParams = Application.getStartupParams();
		final String[] params = startParams.getParams();

		for (final String string : params) {
			if (string.startsWith("bank://")) {
				Logger.debug("Hibiscus mittels BezahlCode-URL handler gestartet. UrlHandler anlegen...");
				Application.getMessagingFactory().registerMessageConsumer(new CliUrlHandler(string)); // NOPMD
			}
		}

	}

	/**
	 * Hier werden alle QR-Code Handler einmal geladen, damit sie in sich in der
	 * Registry anmelden.
	 */
	private void ladeHandler() {

		// Laden der verschiedenen code handler
		touchClass("de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers.BezahlcodeSinglePaymentSepaHandler");
		touchClass("de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers.BezahlcodeSinglePaymentHandler");
		touchClass("de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers.BezahlcodeContactV2Handler");
		touchClass("de.frankmuenster.jameica.qrcodehandler.BcdSepaHandler.BcdSepa_001_SCT");
		touchClass("de.frankmuenster.jameica.qrcodehandler.BcdSepaHandler.BcdSepa_002_SCT");

	}

	/**
	 * Methode die fr einen angegebenen Klassenname ein Class.forName
	 * durchfhrt.
	 * 
	 * @param className
	 *            Name der Klasse die geladen werden soll.
	 */
	private void touchClass(final String className) {

		if (className != null && !className.isEmpty()) {
			try {
				Class.forName(className);
			} catch (ClassNotFoundException e) {
				Logger.debug("QrCodeHandler-Klasse " + className + " nicht gefunden.");
			}
		}
	}

	/**
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#shutDown()
	 */
	@Override
	public void shutDown() {
		Logger.debug("Shuting down Plugin QrCodeScanner.");
		super.shutDown();
	}
}

/***********************************************************
 * History: $Log QrCodeScanner.java,v$ $Date:: 2014-04-01 00:56:03$ Date of last
 * commit
 * 
 * @Revision 1.1 Date: 20.04.2014 Navigationmen umgestellt von plugins-xml zu
 *           Extension ExtensionRegistry.register hinzugefgt.
 * 
 *           1.2 Date: 27.07.2016 CliUrlHandler-Aufruf zum Bearbeiten von
 *           BezahlCode URL in der Kommandozeile hinzugefgt.
 * 
 *********************************************************/
