package de.frankmuenster.jameica.qrscanner.utils;

/**
 * Utility-Klasse fr Validierer (nicht BIC und IBAN).
 * 
 * @author Frank Mnster
 *
 */
@SuppressWarnings("PMD")

public final class Validators {

	/**
	 * &Uuml;berpr&uuml;ft eine IPv4 Adresse. Liefert true wenn sie richtig ist.
	 * 
	 * 
	 * @param ipAddress
	 *            die zu pr&uumlfende IPv4 Adresse
	 * @return true, falls das Argument eine g&uum;ltige IPv4 Adresse ist.
	 * 
	 * @see <a href="http://stackoverflow.com/a/31099148">stackoverflow -
	 *      Validating IPv4 string in Java</a>
	 */
	public static boolean isValidInet4Address(final String ipAddress) {

		if (ipAddress == null || ipAddress.length() < 7 || ipAddress.length() > 15)
			return false;

		try {
			int x = 0;
			int y = ipAddress.indexOf('.');

			if (y == -1 || ipAddress.charAt(x) == '-' || Integer.parseInt(ipAddress.substring(x, y)) > 255)
				return false;

			x = ipAddress.indexOf('.', ++y);
			if (x == -1 || ipAddress.charAt(y) == '-' || Integer.parseInt(ipAddress.substring(y, x)) > 255)
				return false;

			y = ipAddress.indexOf('.', ++x);
			return !(y == -1 || ipAddress.charAt(x) == '-' || Integer.parseInt(ipAddress.substring(x, y)) > 255
					|| ipAddress.charAt(++y) == '-'
					|| Integer.parseInt(ipAddress.substring(y, ipAddress.length())) > 255
					|| ipAddress.charAt(ipAddress.length() - 1) == '.');

		} catch (NumberFormatException e) {
			return false;
		}
	}

}
